#!/usr/bin/env python3

import argparse
import os
import re
import sys
import subprocess
import logging
import shutil
from opal import get_labels
from opal import make_sure_path_exists
from opal_stats import get_image_dir_name
from version import __version__


def get_sorted_list_of_profiles_files(path):
    p = re.compile('^result_[0-9]+__.+\.profile$')
    all_files = os.listdir(path)
    mylist = []
    for file in all_files:
        if not p.match(file):
            continue
        mylist.append(file)
    mylist.sort()
    return mylist


def read_stats(results_dir, images_list):
    time = ''
    memory = ''
    comma = ''
    for image in images_list:
        image_dir_name = get_image_dir_name(image)
        with open(os.path.join(results_dir, image_dir_name, 'runtime_maxmemory.txt')) as f_input:
            # assumes first line is time and second, memory
            time += comma + str(float(f_input.readline().split(' ', 1)[0])/3600)
            memory += comma + str(float(f_input.readline().split(' ', 1)[0])/1024)
        comma = ','
    return time, memory


def preprocess_results(results_dir, images_list):
    index_empty_results = []
    p1 = re.compile('[0-9]+')
    p2 = re.compile('^@[^@]')
    for i, image in enumerate(images_list):
        image_dir_name = get_image_dir_name(image)
        file_path = os.path.join(results_dir, image_dir_name, 'all_results.profile')
        f_output = open(file_path, 'w')
        sorted_files = get_sorted_list_of_profiles_files(os.path.join(results_dir, image_dir_name))

        for file in sorted_files:
            sample_number = p1.search(file).group()
            f_input = open(os.path.join(results_dir, image_dir_name, file))

            for line in f_input:
                if p2.match(line):
                    key, value = line[1:].split(':', 1)
                    if key.upper() == 'SAMPLEID':
                        f_output.write('@{}:{}\n'.format(key, sample_number))
                    else:
                        f_output.write(line)
                else:
                    f_output.write(line)
            f_input.close()
        f_output.close()
        if os.stat(file_path).st_size == 0:
            index_empty_results.append(i)
    return index_empty_results


def get_profiles_list(results_dir, images_list):
    profiles_list = []
    for image in images_list:
        image_dir_name = get_image_dir_name(image)
        profiles_list.append(os.path.join(results_dir, image_dir_name, 'all_results.profile'))
    return profiles_list


def get_logger(output_dir):
    make_sure_path_exists(output_dir)
    logger = logging.getLogger('opal_workflow')
    logger.setLevel(logging.INFO)
    formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
    logging_fh = logging.FileHandler(os.path.join(output_dir, 'log.txt'))
    logging_fh.setFormatter(formatter)
    logger.addHandler(logging_fh)

    logging_stdout = logging.StreamHandler(sys.stdout)
    logging_stdout.setFormatter(formatter)
    logger.addHandler(logging_stdout)
    return logger


def main():
    parser = argparse.ArgumentParser(description='Run bioboxes of profilers with opal_stats and assess results with OPAL', add_help=False)
    group1 = parser.add_argument_group('required arguments')
    group1.add_argument('images', nargs='+', help='Docker images (bioboxes) of profilers')
    group1.add_argument('--input_dir', help='Input directory containing gzipped FASTQ files', required=True)
    group1.add_argument('--output_dir', help='Output directory', required=True)
    group1.add_argument('-g', '--gold_standard_file', help='Gold standard file', required=True)
    group2 = parser.add_argument_group('optional arguments')
    group2.add_argument('--yaml', help='Bioboxes YAML file (default: INPUT_DIR/biobox.yaml)', required=False)
    group2.add_argument("--volume", help='Docker volume', action='append')
    group2.add_argument('-n', '--no_normalization', help='Do not normalize samples', action='store_true')
    group2.add_argument('-p', '--plot_abundances', help='Plot abundances in the gold standard', action='store_true')
    group2.add_argument('-l', '--labels', help='Comma-separated names of profilers to be shown in OPAL', required=False)
    group2.add_argument('-d', '--desc', help='Description for HTML page', required=False)
    group2.add_argument('-v', '--version', action='version', version='%(prog)s ' + __version__)
    group2.add_argument('-h', '--help', action='help', help='Show this help message and exit')
    args = parser.parse_args()

    logger = get_logger(args.output_dir)
    images = args.images[:]
    labels = get_labels(args.labels, images)

    for image in images:
        logger.info('Running {}...'.format(image))
        parameters = [image,
                      '--input_dir=' + args.input_dir,
                      '--output_dir=' + args.output_dir]
        if args.yaml:
            parameters.append('--yaml=' + args.yaml)
        if args.volume:
            for volume in args.volume:
                parameters.append('--volume=' + volume)
        try:
            opal_stats = shutil.which('opal_stats.py', path=os.getcwd())
            if not opal_stats:
                opal_stats = shutil.which('opal_stats.py')
            subprocess.run([sys.executable, opal_stats] + parameters, check=True)
        except subprocess.CalledProcessError:
            logger.error('Error: opal_stats.py returned non-zero exit status for ' + image)
    logger.info('done')

    logger.info('Reading results...')
    index_empty_results = preprocess_results(args.output_dir, images)
    for index in sorted(index_empty_results, reverse=True):
        logger.warning('No results available for ' + images[index])
        del images[index]
        del labels[index]
    profiles_list = get_profiles_list(args.output_dir, images)
    time, memory = read_stats(args.output_dir, images)
    logger.info('done')

    logger.info('Running OPAL...')
    parameters = ['--gold_standard_file=' + args.gold_standard_file,
                  '--time=' + time,
                  '--memory=' + memory,
                  '--output_dir=' + os.path.join(args.output_dir, 'opal_output')]
    if args.no_normalization:
        parameters.append('--no_normalization')
    if args.plot_abundances:
        parameters.append('--plot_abundances')
    parameters.append('--labels=' + ','.join(labels))
    if args.desc:
        parameters.append('--desc=' + args.desc)
    parameters += profiles_list
    opal = shutil.which('opal.py', path=os.getcwd())
    if not opal:
        opal = shutil.which('opal.py')
    subprocess.run([sys.executable, opal] + parameters, check=True)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

from collections import defaultdict
from collections import OrderedDict

RANKS = ['superkingdom', 'phylum', 'class', 'order', 'family', 'genus', 'species', 'strain']
DICT_RANK_TO_INDEX = dict(zip(RANKS, list(range(len(RANKS)))))


def load_names(tax_id_to_rank, names_file_path):
    tax_id_to_name = {}
    with open(names_file_path) as read_handler:
        for line in read_handler:
            if len(line.strip()) == 0:
                continue
            line = line.split('|')
            line = list(map(str.strip, line))
            tax_id = line[0]

            if line[3] == "scientific name":
                tax_id_to_name[tax_id] = line[1]
            else:
                continue

            if tax_id_to_rank[tax_id] == "species" or tax_id_to_rank[tax_id] == "strain":
                names = tax_id_to_name[tax_id].split(" ")
                if len(names) > 2:
                    if tax_id_to_rank[tax_id] == "strain":
                        tax_id_to_name[tax_id] = "{} {} strain".format(names[0], names[1])
                    else:
                        tax_id_to_name[tax_id] = "{} {}".format(names[0], names[1])
    tax_id_to_name[""] = ""
    return tax_id_to_name


def set_strain_rank_for_tax_id(tax_id, tax_id_to_parent, tax_id_to_rank):
    if tax_id_to_rank[tax_id] == "no rank" and tax_id_to_parent[tax_id] in tax_id_to_parent and (
            tax_id_to_rank[tax_id_to_parent[tax_id]] == "species" or
            tax_id_to_rank[tax_id_to_parent[tax_id]] == "subspecies" or
            tax_id_to_rank[tax_id_to_parent[tax_id]] == "strain") or tax_id_to_rank[tax_id] == "subspecies":
        tax_id_to_rank[tax_id] = "strain"
        return True
    return False


def load_tax_info(ncbi_nodes_file):
    tax_id_to_parent = {}
    tax_id_to_rank = {}
    with open(ncbi_nodes_file) as read_handler:
        for line in read_handler:
            if len(line.strip()) == 0:
                continue
            line = line.split('|')
            line = list(map(str.strip, line))
            tax_id = line[0]
            tax_id_to_parent[tax_id] = line[1]
            tax_id_to_rank[tax_id] = line[2]

    for tax_id, rank in tax_id_to_rank.items():
        if not set_strain_rank_for_tax_id(tax_id, tax_id_to_parent, tax_id_to_rank):
            if tax_id_to_parent[tax_id] in tax_id_to_parent:
                set_strain_rank_for_tax_id(tax_id_to_parent[tax_id], tax_id_to_parent, tax_id_to_rank)

    return tax_id_to_parent, tax_id_to_rank


def get_id_path(tax_id, tax_id_to_parent, tax_id_to_rank):
    index = DICT_RANK_TO_INDEX[tax_id_to_rank[tax_id]]
    path = [''] * (index + 1)
    path[index] = tax_id

    id = tax_id
    while id in tax_id_to_parent:
        id = tax_id_to_parent[id]
        if id == '1':
            break
        if tax_id_to_rank[id] not in RANKS:
            continue
        index = DICT_RANK_TO_INDEX[tax_id_to_rank[id]]
        if path[index] == '':
            path[index] = id
        if tax_id_to_rank[id] == "superkingdom":
            break
    return path


def main():
    xx = defaultdict()
    try:
        print(xx['a'])
    except KeyError:
        print("no key")
    exit()



    ncbi_nodes_file = "/home/fmeyer/cami/taxonomy/nodes.dmp"
    ncbi_names_file = "/home/fmeyer/cami/taxonomy/names.dmp"

    tax_id_to_parent, tax_id_to_rank = load_tax_info(ncbi_nodes_file)
    tax_id_to_name = load_names(tax_id_to_rank, ncbi_names_file)

    rank_to_tax_id_to_abundance = defaultdict(dict)
    rank_to_tax_id_to_id_path = defaultdict(dict)

    tax_id_list = []
    with open("/home/fmeyer/Opal/submission2/hmp_gs.tsv") as f_read:
        for line in f_read:
            if len(line.strip()) == 0:
                continue
            tax_id, abundance = line.rstrip('\n').split('\t', 1)
            rank = tax_id_to_rank[tax_id]
            try:
                rank_index = DICT_RANK_TO_INDEX[rank]
            except:
                print("{} {}".format(tax_id, rank))
                exit(1)

            rank_to_tax_id_to_abundance[rank_index][tax_id] = float(abundance) * 100
            tax_id_list.append(tax_id)
            # print("{}\t{}\t{}\t{}".format(taxid, tax_id_to_parent[taxid], abundance, tax_id_to_name[taxid]))
            # print(get_id_path(taxid, tax_id_to_parent, tax_id_to_rank))

    strain_index = DICT_RANK_TO_INDEX["strain"]
    for tax_id in tax_id_list:
        rank_to_tax_id_to_id_path[strain_index][tax_id] = get_id_path(tax_id, tax_id_to_parent, tax_id_to_rank)

    for rank in RANKS[:-1]:
        rank_index = DICT_RANK_TO_INDEX[rank]
        # sort rank_to_tax_id_to_id_path such that tax ids. with the same parent become adjacent
        tax_id_to_id_path_sorted = OrderedDict(sorted(rank_to_tax_id_to_id_path[strain_index].items(), key=lambda t: rank_to_tax_id_to_id_path[strain_index][t[0]][rank_index]))
        tax_id2 = None
        for tax_id in tax_id_to_id_path_sorted:
            tax_id_r = tax_id_to_id_path_sorted[tax_id][rank_index]
            if tax_id_r == '':
                tax_id2 = None
                continue
            if tax_id_r == tax_id2:
                rank_to_tax_id_to_abundance[rank_index][tax_id_r] += rank_to_tax_id_to_abundance[strain_index][tax_id]
            else:
                rank_to_tax_id_to_abundance[rank_index][tax_id_r] = rank_to_tax_id_to_abundance[strain_index][tax_id]
                rank_to_tax_id_to_id_path[rank_index][tax_id_r] = rank_to_tax_id_to_id_path[strain_index][tax_id][:rank_index + 1]
            tax_id2 = tax_id_r

    sample_id = "sampleid"

    print("@SampleID:{}\n@Version:0.9.1\n@Ranks:superkingdom|phylum|class|order|family|genus|species|strain\n\n@@TAXID\tRANK\tTAXPATH\tTAXPATHSN\tPERCENTAGE".format(sample_id))
    for rank in RANKS:
        rank_index = DICT_RANK_TO_INDEX[rank]
        for tax_id in rank_to_tax_id_to_abundance[rank_index]:
            id_path = rank_to_tax_id_to_id_path[rank_index][tax_id]
            name_path = []
            for id in id_path:
                name_path.append(tax_id_to_name[id])
            print("{}\t{}\t{}\t{}\t{}".format(tax_id, tax_id_to_rank[tax_id], "|".join(id_path), "|".join(name_path), rank_to_tax_id_to_abundance[rank_index][tax_id]))


main()



import os
from collections import defaultdict
from src.utils import constants as c
import pandas as pd
import datetime
import numpy as np
import math
import re
from jinja2 import Template
from statistics import median

import matplotlib
matplotlib.use('Agg')
import seaborn as sns
import matplotlib.pyplot as pltx
from matplotlib.colors import rgb2hex
from matplotlib.colors import Normalize

from version import __version__

from bokeh.plotting import figure
from bokeh.layouts import column, row
from bokeh.models.widgets import TableColumn, Slider, Div, Select, Panel, Tabs, CheckboxGroup
from bokeh.models import (DataTable,
                          CustomJS)
from bokeh.embed import components
from bokeh.resources import INLINE
from bokeh.plotting import ColumnDataSource


SUM_OF_SCORES = 'Sum of scores'


def create_heatmap_bar(output_dir):
    fig = pltx.figure(figsize=(2, 2))
    x = np.arange(25).reshape(5, 5)
    cm = sns.diverging_palette(12, 250, sep=30, l=50, n=15, s=90, as_cmap=True)
    ax = sns.heatmap(x, cmap=cm, cbar=False)
    cbar = pltx.colorbar(ax.get_children()[0], orientation='horizontal')

    # hide border
    cbar.outline.set_visible(False)

    # hide ticks
    cbar.set_ticks([])

    # hide heatmap
    pltx.gca().set_visible(False)

    fig.savefig(os.path.join(output_dir, 'heatmap_bar.png'), dpi=100, format='png', bbox_inches='tight', pad_inches=-.001, transparent=True)
    pltx.close(fig)


def create_title_div(id, name, info):
    div = Div(text="""<div style="text-align:left;font-size: 20pt;font-weight: bold;">{1}<span style="float: right;font-size: 10pt;font-weight:normal;">{2}</span>""".format(id, name, info), css_classes=['bk-width-auto']) # width=DIV_WIDTH, height=DIV_HEIGHT)
    return div


def get_columns(labels, current_columns):
        columns = [c.GS]
        for label in labels:
            if label in current_columns:
                columns.append(label)
        return columns


def get_rank_to_sample_pd(pd_metrics):
    rank_to_sample_pd = defaultdict(dict)
    pd_copy = pd_metrics.copy()

    # label ranks, so that the rows don't get lost
    pd_copy.loc[pd_copy[pd.isnull(pd_copy['rank'])].index, 'rank'] = 'rank independent'

    pd_copy = pd_copy[-pd_copy['metric'].isin(['time', 'memory'])]

    # transform table
    pd_grouped = pd_copy.pivot_table(index=['rank', 'tool', 'sample'], columns='metric', values='value')

    # add sample "(average over samples)"
    pd_mean_over_samples = pd_grouped.groupby(['rank', 'tool'], sort=False).mean()
    # convert means to string to store '<mean> (<standard error>)'
    pd_mean_over_samples_str = pd_mean_over_samples.astype(str)
    pd_sem_over_samples = pd_grouped.groupby(['rank', 'tool'], sort=False).sem()

    # add standard error to "(average over samples)"
    df_columns = pd_sem_over_samples.columns.values
    for index, row in pd_sem_over_samples.iterrows():
        for j, item in enumerate(row):
            sem_value = pd_sem_over_samples.at[index, df_columns[j]]
            mean_value = '{:>.{precision}g}'.format(pd_mean_over_samples.at[index, df_columns[j]], precision=3)
            # if standard error is nan because there is only one sample, then just copy mean
            if np.isnan(sem_value):
                pd_mean_over_samples_str.at[index, df_columns[j]] = mean_value
            else:
                sem_value = '{:>.{precision}g}'.format(sem_value, precision=3)
                sem_value = '<div class="tooltip tooltip-sem">{}<span class="tooltiptext">standard error: {}</span></div>'.format(sem_value, sem_value)
                pd_mean_over_samples_str.at[index, df_columns[j]] = "{} ({})".format(mean_value, sem_value)
    pd_mean_over_samples_str['sample'] = '(average over samples)'
    pd_mean_over_samples_str = pd_mean_over_samples_str.reset_index().set_index(['rank', 'tool', 'sample'])

    pd_grouped = pd.concat([pd_grouped, pd_mean_over_samples_str])

    # copy unifrac values to every taxonomic rank
    pd_grouped_copy = pd_grouped.copy()
    for index, row in pd_grouped_copy.iterrows():
        pd_grouped.loc[index][c.UNIFRAC] = pd_grouped.loc[('rank independent', index[1], index[2])][c.UNIFRAC]
        pd_grouped.loc[index][c.UNW_UNIFRAC] = pd_grouped.loc[('rank independent', index[1], index[2])][c.UNW_UNIFRAC]

    for (rank, sample), g in pd_grouped.groupby(['rank', 'sample']):
        rank_to_sample_pd[rank][sample] = g.reset_index().rename(columns={'tool': 'Tool'}).drop(['rank', 'sample'], axis=1).set_index('Tool').T
        # drop all metrics except unifrac for 'rank independent'
        if rank == 'rank independent':
            rank_to_sample_pd[rank][sample] = rank_to_sample_pd[rank][sample].drop(c.ALL_METRICS[2:])
    return rank_to_sample_pd


def get_formatted_pd_rankings(pd_rankings):
    df_list = []
    df_list_unsorted_pos = []
    metrics_list = []
    for metric, g in pd_rankings.groupby(level=0):
        metrics_list.append(metric)
        df = g.reset_index().sort_values('position')
        df2 = g.reset_index()
        df_list.append(pd.DataFrame({metric: df['tool'].tolist(), 'score' + metric: df['position'].tolist()}))
        df_list_unsorted_pos.append(pd.DataFrame({metric: df2['tool'].tolist(), 'score' + metric: df2['position'].tolist()}))

    df_sum = pd_rankings.groupby(['tool'])['position'].sum().reset_index().sort_values('position')
    df_sum_unsorted_pos = pd_rankings.groupby(['tool'])['position'].sum().reset_index()
    df_list.append(
        pd.DataFrame({SUM_OF_SCORES: df_sum['tool'].tolist(), 'score' + SUM_OF_SCORES: df_sum['position'].tolist()}))
    df_list_unsorted_pos.append(
        pd.DataFrame({SUM_OF_SCORES: df_sum_unsorted_pos['tool'].tolist(), 'score' + SUM_OF_SCORES: df_sum_unsorted_pos['position'].tolist()}))

    pd_show = pd.concat(df_list, axis=1)
    pd_show_unsorted_pos = pd.concat(df_list_unsorted_pos, axis=1)
    return pd_show, pd_show_unsorted_pos


def create_rankings_html(pd_rankings, ranks_scored):
    pd_show, pd_show_unsorted_pos = get_formatted_pd_rankings(pd_rankings)

    table_source = ColumnDataSource(pd_show)

    columns = [
        TableColumn(field=SUM_OF_SCORES, title=SUM_OF_SCORES, sortable=False),
        TableColumn(field='score' + SUM_OF_SCORES, title='', width=50),
        TableColumn(field=c.RECALL, title=c.RECALL, sortable=False),
        TableColumn(field='score' + c.RECALL, title='', width=50),
        TableColumn(field=c.PRECISION, title=c.PRECISION, sortable=False),
        TableColumn(field='score' + c.PRECISION, title='', width=50),
        TableColumn(field=c.L1NORM, title=c.L1NORM, sortable=False),
        TableColumn(field='score' + c.L1NORM, title='', width=50),
        TableColumn(field=c.UNIFRAC, title=c.UNIFRAC, sortable=False),
        TableColumn(field='score' + c.UNIFRAC, title='', width=50),
    ]
    data_table = DataTable(source=table_source, columns=columns, width=800, height=25 + len(pd_show) * 25)

    top = [float(x) for x in pd_show_unsorted_pos['score' + SUM_OF_SCORES]]
    source = ColumnDataSource(data=dict(x=pd_show_unsorted_pos[SUM_OF_SCORES].tolist(),
                                        top=top,
                                        recall=pd_show_unsorted_pos['score' + c.RECALL],
                                        precision=pd_show_unsorted_pos['score' + c.PRECISION],
                                        l1norm=pd_show_unsorted_pos['score' + c.L1NORM],
                                        unifrac=pd_show_unsorted_pos['score' + c.UNIFRAC]))

    callback = CustomJS(args=dict(source=source), code="""
        var data = source.data;
        var wrecall = weight_recall.value;
        var wprecision = weight_precision.value;
        var wl1norm = weight_l1norm.value;
        var wunifrac = weight_unifrac.value;
        topx = data['top'];
        recall = data['recall'];
        precision = data['precision'];
        l1norm = data['l1norm'];
        unifrac = data['unifrac'];
        
        
        for (i = 0; i < topx.length; i++) {
            topx[i] = recall[i] * wrecall + precision[i] * wprecision + l1norm[i] * wl1norm + unifrac[i] * wunifrac;
        }
        
        source.change.emit();
    """)

    weight_recall = Slider(start=0, end=10, value=1, step=.1, title=c.RECALL + " weight", callback=callback)
    callback.args["weight_recall"] = weight_recall

    weight_precision = Slider(start=0, end=10, value=1, step=.1, title=c.PRECISION + " weight", callback=callback)
    callback.args["weight_precision"] = weight_precision

    weight_l1norm = Slider(start=0, end=10, value=1, step=.1, title=c.L1NORM + " weight", callback=callback)
    callback.args["weight_l1norm"] = weight_l1norm

    weight_unifrac = Slider(start=0, end=10, value=1, step=.1, title=c.UNIFRAC + " weight", callback=callback)
    callback.args["weight_unifrac"] = weight_unifrac

    p = figure(x_range=pd_show_unsorted_pos[SUM_OF_SCORES].tolist(), plot_width=800, plot_height=400, title=SUM_OF_SCORES + " - lower is better")
    p.vbar(x='x', top='top', source=source, width=0.5, bottom=0, color="firebrick")

    col_rankings = column([Div(text="<font color='navy'><u>Hint 1:</u> click on the columns of scores for sorting.</font>", style={"width": "600px", "margin-bottom": "0px"}),
                           Div(text="Taxonomic ranks scored: " + ", ".join(ranks_scored), style={"width": "600px", "margin-bottom": "0px"}),
                           data_table,
                           Div(text="<font color='navy'><u>Hint 2:</u> slide the bars to change the weight of the metrics.</font>", style={"width": "500px", "margin-top": "18px"}),
                           row(weight_recall, weight_precision),
                           row(weight_l1norm, weight_unifrac),
                           p], css_classes=['bk-padding-top'])
    return col_rankings


def get_colors_and_ranges(name, all_values, df_metrics):
    color1 = 'dodgerblue'
    color2 = 'red'
    hue1 = 12
    hue2 = 240
    if name == c.PRECISION or name == c.RECALL or name == c.F1_SCORE or name == c.JACCARD:
        return color1, color2, hue1, hue2, 0, 1
    if name == c.FP or name == c.UNIFRAC or name == c.UNW_UNIFRAC:
        return color2, color1, hue2, hue1, 0, max(all_values)
    if name == c.TP:
        return color1, color2, hue1, hue2, 0, max(all_values)
    if name == c.FN:
        fn_values = df_metrics.loc[c.TP, ].values
        # convert "<mean> (<standard error>)" to float of <mean>
        if len(fn_values) > 0 and isinstance(fn_values[0], str):
            fn_values = [float(x.split(' ')[0]) for x in fn_values]
        return color2, color1, hue2, hue1, 0, max(fn_values)
    if name == c.L1NORM:
        return color2, color1, hue2, hue1, 0, 2
    if name == c.BRAY_CURTIS:
        return color2, color1, hue2, hue1, 0, 1
    return color1, color2, hue1, hue2, max(all_values), min(all_values)


class MidpointNormalize(Normalize):
    def __init__(self, vmin=None, vmax=None, midpoint=None, clip=False):
        self.midpoint = midpoint
        Normalize.__init__(self, vmin, vmax, clip)

    def __call__(self, value, clip=None):
        x, y = [self.vmin, self.midpoint, self.vmax], [0, 0.5, 1]
        return np.ma.masked_array(np.interp(value, x, y))


def get_heatmap_colors(pd_series, **args):
    values = pd_series.tolist()

    # convert "<mean> (<standard error>)" to float of <mean>
    if len(values) > 0 and isinstance(values[0], str):
        values = [float(x.split(' ')[0]) for x in values]
    all_values = values[:]

    dropped_gs = False
    if pd_series.index[0] == c.GS:
        pd_series.drop(c.GS)
        values = values[1:]
        dropped_gs = True
    if len(values) == 0:
        return ['']

    notnan_values = [x for x in values if isinstance(x, (float, int)) and not np.isnan(x)]
    notnan_all_values = [x for x in all_values if isinstance(x, (float, int)) and not np.isnan(x)]
    if len(notnan_values) == 0:
        red = 'background-color: red'
        return [red for x in values] if not dropped_gs else [''] + [red for x in values]

    color1, color2, hue1, hue2, min_value, max_value = get_colors_and_ranges(pd_series.name, all_values, args["df_metrics"])

    cm = sns.diverging_palette(hue1, hue2, sep=50, l=80, n=15, s=90, as_cmap=True)
    norm = MidpointNormalize(vmin=min_value, vmax=max_value, midpoint=median(notnan_all_values))

    normed = norm([round(x, 3) if not math.isnan(x) else max_value for x in values])
    heatmap_colors = [rgb2hex(x) for x in pltx.cm.get_cmap(cm)(normed)]
    return_colors = []
    for val, x in zip(values, heatmap_colors):
        if val == min_value:
            return_colors.append('background-color: {}'. format(color2))
        elif val == max_value:
            return_colors.append('background-color: {}'. format(color1))
        elif math.isnan(val):
            return_colors.append('background-color: red')
        else:
            return_colors.append('background-color: {}'. format(x))

    if dropped_gs:
        return [''] + return_colors
    else:
        return return_colors


def create_metrics_table(pd_metrics, labels, sample_ids_list):
    rank_to_sample_pd = get_rank_to_sample_pd(pd_metrics)

    all_sample_ids = sample_ids_list[:]
    all_sample_ids.insert(0, '(average over samples)')

    presence_metrics = [c.RECALL, c.PRECISION, c.F1_SCORE, c.TP, c.FP, c.FN, c.JACCARD]
    estimates_metrics = [c.UNIFRAC, c.UNW_UNIFRAC, c.L1NORM, c.BRAY_CURTIS]
    alpha_diversity_metics = [c.OTUS, c.SHANNON_DIVERSITY, c.SHANNON_EQUIT]
    rank_independent_metrics = [c.UNIFRAC, c.UNW_UNIFRAC]
    all_metrics = [presence_metrics, estimates_metrics, alpha_diversity_metics]

    presence_metrics_label = 'Presence/absence of taxa'
    estimates_metrics_label = 'Abundance estimates'
    alpha_diversity_metics = 'Alpha diversity'
    all_metrics_labels = [presence_metrics_label, estimates_metrics_label, alpha_diversity_metics]

    styles = [{'selector': 'td', 'props': [('width', '100pt')]},
              {'selector': 'th', 'props': [('width', '100pt'), ('text-align', 'left')]},
              {'selector': 'th:nth-child(1)', 'props': [('width', '120pt'), ('font-weight', 'normal')]},
              {'selector': '', 'props': [('width', 'max-content'), ('width', '-moz-max-content'), ('border-top', '1px solid lightgray'), ('border-spacing', '0px')]},
              {'selector': 'expand-toggle:checked ~ * .data', 'props': [('background-color', 'white !important')]}]
    styles_hidden_thead = styles + [{'selector': 'thead', 'props': [('display', 'none')]}]

    d = {c.RECALL: '<div class="tooltip">{}<span class="tooltiptext">{}: {}</span></div>'.format(c.RECALL, c.RECALL, c.TOOLTIP_RECALL),
         c.PRECISION: '<div class="tooltip">{}<span class="tooltiptext">{}: {}</span></div>'.format(c.PRECISION, c.PRECISION, c.TOOLTIP_PRECISION),
         c.F1_SCORE: '<div class="tooltip">{}<span class="tooltiptext">{}: {}</span></div>'.format(c.F1_SCORE, c.F1_SCORE, c.TOOLTIP_F1_SCORE),
         c.TP: '<div class="tooltip">{}<span class="tooltiptext">{}: {}</span></div>'.format(c.TP, c.TP, c.TOOLTIP_TP),
         c.FP: '<div class="tooltip">{}<span class="tooltiptext">{}: {}</span></div>'.format(c.FP, c.FP, c.TOOLTIP_FP),
         c.FN: '<div class="tooltip">{}<span class="tooltiptext">{}: {}</span></div>'.format(c.FN, c.FN, c.TOOLTIP_FN),
         c.JACCARD: '<div class="tooltip">{}<span class="tooltiptext">{}: {}</span></div>'.format(c.JACCARD, c.JACCARD, c.TOOLTIP_JACCARD),
         c.UNIFRAC: '<div class="tooltip">{}<span class="tooltiptext">{}: {}</span></div>'.format(c.UNIFRAC, c.UNIFRAC, c.TOOLTIP_UNIFRAC),
         c.UNW_UNIFRAC: '<div class="tooltip">{}<span class="tooltiptext">{}: {}</span></div>'.format(c.UNW_UNIFRAC, c.UNW_UNIFRAC, c.TOOLTIP_UNW_UNIFRAC),
         c.L1NORM: '<div class="tooltip">{}<span class="tooltiptext">{}: {}</span></div>'.format(c.L1NORM, c.L1NORM, c.TOOLTIP_L1NORM),
         c.BRAY_CURTIS: '<div class="tooltip">{}<span class="tooltiptext">{}: {}</span></div>'.format(c.BRAY_CURTIS, c.BRAY_CURTIS, c.TOOLTIP_BRAY_CURTIS),
         c.OTUS: '<div class="tooltip">{}<span class="tooltiptext">{}: {}</span></div>'.format(c.OTUS, c.OTUS, c.TOOLTIP_OTUS),
         c.SHANNON_DIVERSITY: '<div class="tooltip">{}<span class="tooltiptext">{}: {}</span></div>'.format(c.SHANNON_DIVERSITY, c.SHANNON_DIVERSITY, c.TOOLTIP_SHANNON_DIVERSITY),
         c.SHANNON_EQUIT: '<div class="tooltip">{}<span class="tooltiptext">{}: {}</span></div>'.format(c.SHANNON_EQUIT, c.SHANNON_EQUIT, c.TOOLTIP_SHANNON_EQUIT)}
    pattern = re.compile('|'.join(map(re.escape, d)))

    def translate(match):
        return d[match.group(0)]

    rank_to_sample_to_html = defaultdict(list)
    for rank in rank_to_sample_pd:
        for sample_id in all_sample_ids:
            if sample_id not in rank_to_sample_pd[rank]:
                rank_to_sample_to_html[rank].append("")
                continue
            mydf = rank_to_sample_pd[rank][sample_id]
            mydf.index.name = None
            html = ''
            first_metrics = True
            for metrics, metrics_label in zip(all_metrics, all_metrics_labels):
                if rank == 'rank independent':
                    metrics = rank_independent_metrics
                    metrics_label = estimates_metrics_label
                html += '<p style="margin-bottom: auto"><b>{}</b></p>'.format(metrics_label)
                mydf_metrics = mydf.loc[metrics]

                sorted_columns = get_columns(labels, mydf_metrics.columns.tolist())
                mydf_metrics = mydf_metrics.loc[:, sorted_columns]

                if first_metrics:
                    this_style = styles
                else:
                    this_style = styles_hidden_thead
                if metrics_label == presence_metrics_label or metrics_label == estimates_metrics_label:
                    html += mydf_metrics.style.apply(get_heatmap_colors, df_metrics=mydf_metrics, axis=1).set_precision(3).set_table_styles(this_style).render()
                else:
                    html += mydf_metrics.style.set_precision(3).set_table_styles(this_style).render()
                if rank == 'rank independent':
                    break
                first_metrics = False
            html = pattern.sub(translate, html)
            rank_to_sample_to_html[rank].append(html)

    mytable1 = Div(text="""<div>{}</div>""".format(rank_to_sample_to_html[c.ALL_RANKS[0]][0]), css_classes=['bk-width-auto'])

    select_rank = Select(title="Taxonomic rank:", value=c.ALL_RANKS[0], options=c.ALL_RANKS + ['rank independent'], css_classes=['bk-fit-content'])

    select_sample = Select(title="Sample:", value='0', options=list(zip(map(str, range(len(all_sample_ids))), all_sample_ids)), css_classes=['bk-fit-content'])

    source = ColumnDataSource(data=rank_to_sample_to_html)

    select_rank_sample_callback = CustomJS(args=dict(source=source), code="""
        mytable1.text = source.data[select_rank.value][select_sample.value];
    """)
    select_rank.js_on_change('value', select_rank_sample_callback)
    select_sample.js_on_change('value', select_rank_sample_callback)
    select_rank_sample_callback.args["select_rank"] = select_rank
    select_rank_sample_callback.args["select_sample"] = select_sample
    select_rank_sample_callback.args["mytable1"] = mytable1

    heatmap_legend = '<img src="heatmap_bar.png" /><div style="text-align:left;font-size: 11px;">Worst<span style="float:right;">Best</span><span style="margin-right: 36px;float:right;">Median</span></div>'
    heatmap_legend_div = Div(text=heatmap_legend, style={"width": "155px", "margin-bottom": "-10px"})
    # <input type="checkbox" id="expand-toggle" /><label for="expand-toggle" id="expand-btn">Toggle</label>
    return select_sample, select_rank, heatmap_legend_div, mytable1


def create_alpha_diversity_tab():
    imgs_shannon = '<img src="plot_shannon.png"/><img src="plot_shannon_diff.png"/>'
    div_plots_shannon = Div(text=imgs_shannon, css_classes=['bk-width-auto'])
    shannon_column = column(div_plots_shannon, sizing_mode='scale_width', css_classes=['bk-width-auto', 'bk-width-auto-main'])
    shannon_panel = Panel(child=shannon_column, title='Shannon')
    tabs_plots = Tabs(tabs=[shannon_panel], css_classes=['bk-tabs-margin', 'bk-tabs-margin-lr'])
    return tabs_plots


def create_plots_html(plots_list):
    message_no_spdplot = 'Spider plots of performance require at least 3 profiles.'

    text = '<img src="spider_plot.png" />' if 'spider_plot' in plots_list else message_no_spdplot
    plot1 = Panel(child=Div(text=text), title='Relative performance')

    text = '<img src="spider_plot_recall_precision.png" />' if 'spider_plot_recall_precision' in plots_list else message_no_spdplot
    plot2 = Panel(child=Div(text=text), title='Absolute performance')

    tabs_plots = Tabs(tabs=[plot1, plot2], width=780, css_classes=['bk-tabs-margin'])
    return tabs_plots


def create_beta_diversity_tab(labels, plots_list):
    rank_to_img = {rank: [''] for rank in c.ALL_RANKS}
    for rank in c.ALL_RANKS:
        for label in labels:
            file = os.path.join("by_tool", label, 'beta_diversity_bc_' + rank)
            if file in plots_list:
                rank_to_img[rank][0] = rank_to_img[rank][0] + '<img src=' + '"' + file + '.png' + '"' + '/>'
    div_plots = Div(text=rank_to_img[c.SPECIES][0], css_classes=['bk-width-auto'])

    source = ColumnDataSource(data=rank_to_img)

    select2_rank_sample_callback = CustomJS(args=dict(source=source), code="""
        div_plots.text = source.data[select2_rank.value][0];
    """)

    select2_rank = Select(title="Taxonomic rank:", value=c.SPECIES, options=c.ALL_RANKS, css_classes=['bk-fit-content'])
    select2_rank.js_on_change('value', select2_rank_sample_callback)
    select2_rank_sample_callback.args["select2_rank"] = select2_rank
    select2_rank_sample_callback.args["div_plots"] = div_plots

    beta_div_column = column(select2_rank, div_plots, sizing_mode='scale_width', css_classes=['bk-width-auto', 'bk-width-auto-main'])
    return beta_div_column


def create_gs_tab(plots_list, tabs_list):
    # Rarefaction curves panel
    imgs = '<img src="gold_standard/rarefaction_curves.png"/><img src="gold_standard/rarefaction_curves_log_scale.png"/>'
    div_plots_rarefaction = Div(text=imgs, css_classes=['bk-width-auto'])
    div_plots_text = Div(text="<div style='margin-top:18px; margin-bottom:0px;'><font color='navy'><ul style='list-style-type:square;margin-bottom:0;margin-top:0;'><li>OPAL always assumes that the samples are from the same environment.</li><li>Dotted lines are accumulation curves.</li></ul></font></div>", css_classes=['bk-width-auto'])
    gs_column_rarefaction = column(div_plots_text, div_plots_rarefaction, sizing_mode='scale_width', css_classes=['bk-width-auto', 'bk-width-auto-main'])
    rarefaction_panel = Panel(child=gs_column_rarefaction, title="Rarefaction curves")

    # Proportions panel
    imgs_proportions = ''
    for rank in c.ALL_RANKS:
        if os.path.join('gold_standard', rank) in plots_list:
            fig_name = 'gold_standard/' + rank
            imgs_proportions = imgs_proportions + '<img src="' + fig_name + '.png" onclick="showlegend(this, \'' + rank + '_legend\')" class="proportions"/>'
            imgs_proportions = imgs_proportions + '<img src="' + fig_name + '_legend.png" style="visibility:hidden;display:none;" id="' + rank + '_legend" class="legend">'
    if len(imgs_proportions) > 0:
        imgs_proportions = "<div style='margin-top:18px; margin-bottom:16px;'><font color='navy'><u>Hint:</u> click on a plot for its legend and drag it around the screen as necessary. Click on the same plot again to hide the legend.</font></div>" + imgs_proportions
        div_plots = Div(text=imgs_proportions, css_classes=['bk-width-auto'])
        gs_column_prop = column(div_plots, sizing_mode='scale_width', css_classes=['bk-width-auto', 'bk-width-auto-main'])
        proportions_panel = Panel(child=gs_column_prop, title="Proportions")

        tabs_plots = Tabs(tabs=[proportions_panel, rarefaction_panel], css_classes=['bk-tabs-margin', 'bk-tabs-margin-lr'])
        tabs_list.append(Panel(child=tabs_plots, title="Gold standard"))
    else:
        tabs_plots = Tabs(tabs=[rarefaction_panel], css_classes=['bk-tabs-margin', 'bk-tabs-margin-lr'])
        tabs_list.append(Panel(child=tabs_plots, title="Gold standard"))


def create_computing_efficiency_tab(pd_metrics, plots_list, tabs_list):
    if 'time_memory' not in plots_list:
        return

    styles = [{'selector': 'td', 'props': [('width', '77pt'), ('padding-left', '10px')]},
              {'selector': 'th', 'props': [('width', 'auto'), ('text-align', 'left'), ('padding-left', '10px')]},
              {'selector': '', 'props': [('width', 'max-content'), ('width', '-moz-max-content'), ('border-spacing', '0px')]}]
    df = pd_metrics[pd_metrics['metric'].isin(['time', 'memory'])]
    df = df.pivot_table(index=['tool'], columns='metric', values='value').reset_index().sort_values(by=['time'])
    df = df[['tool', 'time', 'memory']]
    df.rename(index=str, columns={'tool': 'Tool', 'time': 'Time (hours)', 'memory': 'Memory (GB)'}, inplace=True)

    html = df.style.set_table_styles(styles).hide_index().render()
    div_html = Div(text=html, css_classes=['bk-width-auto', 'bk-inline-block'])

    div_time_memory = Div(text='<img src="time_memory.png"/>', css_classes=['bk-width-auto', 'bk-inline-block'])
    column_time_memory = row(div_html, div_time_memory, sizing_mode='scale_width', css_classes=['bk-width-auto', 'bk-width-auto-main', 'bk-inline-block'])
    tabs_list.append(Panel(child=column_time_memory, title="Computing efficiency"))


def create_html(pd_rankings, ranks_scored, pd_metrics, labels, sample_ids_list, plots_list, output_dir, desc_text):
    # col_rankings = create_rankings_html(pd_rankings, ranks_scored)

    create_heatmap_bar(output_dir)

    select_sample, select_rank, heatmap_legend_div, mytable1 = create_metrics_table(pd_metrics, labels, sample_ids_list)

    tabs_plots = create_plots_html(plots_list)

    metrics_row = row(column(row(select_sample, select_rank), heatmap_legend_div, mytable1), column(tabs_plots))

    # beta_div_column = create_beta_diversity_tab(labels, plots_list)

    tabs_list = [Panel(child=metrics_row, title="Metrics")]

    # tabs_list = [Panel(child=metrics_row, title="Metrics"),
    #              Panel(child=col_rankings, title="Rankings"),
    #              Panel(child=create_alpha_diversity_tab(), title="Alpha diversity"),
    #              Panel(child=beta_div_column, title="Beta diversity")]

    # create_computing_efficiency_tab(pd_metrics, plots_list, tabs_list)

    # create_gs_tab(plots_list, tabs_list)

    # tabs = Tabs(tabs=tabs_list, css_classes=['bk-tabs-margin'])
    tabs = Tabs(tabs=tabs_list)

    title = create_title_div("main", "OPAL: Open-community Profiling Assessment tooL", " produced on {0} with OPAL version {1} ".format(
            datetime.datetime.now().strftime("%Y-%m-%d %H:%M"), __version__))

    template = Template('''<!DOCTYPE html>
        <html lang="en">
            <head>
                <meta charset="utf-8">
                <title>OPAL: Open-community Profiling Assessment tooL</title>
                {{ js_resources }}
                {{ css_resources }}
                <style>
                .tooltip {
                    position: relative;
                    display: inline-block;
                    border-bottom: 1px dashed lightgray;
                    cursor: help;
                }
                .tooltip-sem {
                    border-bottom: 1px dotted black;
                }
                .tooltip .tooltiptext {
                    visibility: hidden;
                    width: 260px;
                    background-color: #555;
                    color: #fff;
                    text-align: center;
                    border-radius: 6px;
                    padding: 5px 0;
                    position: absolute;
                    z-index: 1;
                    bottom: 125%;
                    left: 50%;
                    margin-left: -60px;
                    opacity: 0;
                }
                .tooltip .tooltiptext::after {
                    content: "";
                    position: absolute;
                    top: 100%;
                    left: 50%;
                    margin-left: -5px;
                    border-width: 5px;
                    border-style: solid;
                    border-color: #555 transparent transparent transparent;
                }
                .tooltip:hover .tooltiptext {
                    visibility: visible;
                    opacity: 1;
                }
                .proportions {
                    cursor: pointer;
                }
                .legend {
                    position:absolute;
                    cursor: move;
                    z-index: 1;
                }
                </style>
            </head>
            <body>
                {{ div }}
                {{ script }}
                <script>
                    showlegend = function(img, elementid){
                        var x = document.getElementById(elementid);
                        if (x.style.visibility == 'visible') {
                            x.style.visibility = 'hidden';
                            x.style.display = 'none';
                        } else {
                            if (!x.style.top) {
                                x.style.top = img.offsetTop.toString().concat('px');
                            }
                            if (!x.style.left) {
                                x.style.left = img.offsetLeft.toString().concat('px');
                            }
                            x.style.visibility = 'visible';
                            x.style.display = 'initial';
                        }
                    }
                    function startDrag(e) {
                        if (!e) {
                            var e = window.event;
                        }
                        targ = e.target ? e.target : e.srcElement;
                        if (targ.className != 'legend') {return};
                        offsetX = e.clientX;
                        offsetY = e.clientY;
                        coordX = parseInt(targ.style.left);
                        coordY = parseInt(targ.style.top);
                        drag = true;
                        document.onmousemove=dragDiv;
                        return false;
                    }
                    function dragDiv(e) {
                        if (!drag) {return};
                        if (!e) { var e= window.event};
                        targ.style.left=coordX+e.clientX-offsetX+'px';
                        targ.style.top=coordY+e.clientY-offsetY+'px';
                        return false;
                    }
                    function stopDrag() {
                        drag=false;
                    }
                    window.onload = function() {
                        document.onmousedown = startDrag;
                        document.onmouseup = stopDrag;
                    }
                </script>
            </body>
        </html>
        ''')

    # if desc_text:
    #     data_desc_div = Div(text="""<div style="text-align:left;font-size: 11pt;font-weight: bold;">{}""".format(desc_text), css_classes=['bk-width-auto'])
    #     html_columns = column(title, data_desc_div, tabs, sizing_mode='scale_width', css_classes=['bk-width-auto-main'])
    # else:
    html_columns = column(title, tabs)

    script, div = components(html_columns)
    js_resources = INLINE.render_js()
    css_resources = INLINE.render_css()

    html = template.render(js_resources=js_resources,
            css_resources=css_resources,
            script=script,
            div=div)

    with open(os.path.join(output_dir, "results.html"), 'w') as f:
        f.write(html)



# This is a collection of scripts that will allow manipulation of CAMI profiling files
import sys
import copy
import numpy as np
import timeit


class Profile(object):
    def __init__(self, sample_metadata=None, profile=None):
        self.sample_metadata = sample_metadata
        self.profile = profile
        self._data = dict()
        # Stick in the root node just to make sure everything is consistent
        self._data["-1"] = dict()
        self._data["-1"]["rank"] = None
        self._data["-1"]["tax_path"] = list()
        self._data["-1"]["tax_path_sn"] = list()
        self._data["-1"]["abundance"] = 0
        self._data["-1"]["descendants"] = list()
        self._data["-1"]["branch_length"] = 0
        self._header = list()
        self._tax_id_pos = None
        self._rank_pos = None
        self._tax_path_pos = None
        self._tax_path_sn_pos = None
        self._abundance_pos = None
        self._eps = .0000000000000001  # This is to act like zero, ignore any lines with abundance below this quantity
        self._all_keys = ["-1"]
        self._merged_flag = False
        self.parse_file()

    def parse_file(self):
        _data = self._data
        _all_keys = self._all_keys
        _header = self._header
        for k, v in self.sample_metadata.items():
            _header.append('{}:{}'.format(k, v))
        for prediction in self.profile:
            tax_id = prediction.taxid.strip()
            _all_keys.append(prediction.taxid)
            tax_path = prediction.taxpath.strip().split("|")  # this will be a list, join up late

            if tax_id in _data:  # If this tax_id is already present, add the abundance. NOT CHECKING FOR CONSISTENCY WITH PATH
                _data[tax_id]["abundance"] += prediction.percentage
            else:
                _data[tax_id] = dict()
                _data[tax_id]["abundance"] = prediction.percentage

            if not (prediction.taxpathsn is None): # might not be present
                _data[tax_id]["tax_path_sn"] = prediction.taxpathsn.strip().split("|")  # this will be a list, join up later

            _data[tax_id]["tax_path"] = tax_path
            _data[tax_id]["rank"] = prediction.rank.strip()

            # Find the ancestor
            if len(tax_path) <= 1:
                _data[tax_id]["ancestor"] = "-1"  # no ancestor, it's a root
                _data[tax_id]["branch_length"] = 1
                ancestor = "-1"
            else:
                ancestor = tax_path[-2]
                _data[tax_id]["branch_length"] = 1
                i = -3
                while ancestor is "" or ancestor == tax_id:  # if it's a blank or repeated, go up until finding ancestor
                    ancestor = tax_path[i]
                    _data[tax_id]["branch_length"] += 1
                    i -= 1
                    if i + len(tax_path) < 0:  # no ancestor available, manually set to -1 (the root)
                        ancestor = "-1"
                _data[tax_id]["ancestor"] = ancestor

            # Create a placeholder descendant key initialized to [], just so each tax_id has a descendant key associated to it
            if "descendants" not in _data[tax_id]:  # if this tax_id doesn't have a descendant list,
                _data[tax_id]["descendants"] = list()  # initialize to empty list
            # add the descendants
            if ancestor in _data:  # see if the ancestor is in the data so we can add this entry as a descendant
                if "descendants" not in _data[ancestor]:  # if it's not present, create the descendant list
                    _data[ancestor]["descendants"] = list()
                _data[ancestor]["descendants"].append(
                    tax_id)  # since ancestor is an ancestor, add this descendant to it
            else:  # if it's not already in the data, create the entry
                _data[ancestor] = dict()
                _data[ancestor]["descendants"] = list()
                _data[ancestor]["descendants"].append(tax_id)
                _data[ancestor]["abundance"] = 0.0
        # only fix if need be
        # if all_tax_ids.intersection(_all_keys):
        self._delete_missing()  # make sure there aren't any missing internal nodes

    def _delete_missing(self):
        # This is really only useful for MetaPhlAn profiles, which due to the infrequently updated taxonomy, contain
        # many missing internal nodes, just delete the tax_id (make them "") and adjust branch lengths accordingly
        _data = self._data
        _good_keys = self._all_keys
        current_keys = list(_data)  # .keys()
        bad_keys = []
        # Remove the bad keys from the dictionary
        for key in current_keys:  # go through the current keys
            if key not in _good_keys:  # if it's not a good key
                bad_keys.append(key)  # store the bad key
                del _data[key]  # delete the entry
        # Get the bad keys in the intermediate places
        for key in _good_keys:
            tax_path = _data[key]["tax_path"]
            for tax_id in tax_path:  # look in full taxpath
                if tax_id not in _good_keys:  # if it's a bad key
                    if tax_id not in bad_keys:  # and it's not already in the list
                        bad_keys.append(tax_id)  # add it to the list
        # Don't regard the blank tax_id
        if '' in bad_keys:
            bad_keys.pop(bad_keys.index(''))
        for key in _good_keys:
            # branch_length = _data[key]["branch_length"]
            tax_path = _data[key]["tax_path"]
            # tax_path_sn = _data[key]["tax_path_sn"]
            descendants = _data[key]["descendants"]
            # ancestor = _data[key]["ancestor"]
            for descendant in descendants:
                if descendant in bad_keys:
                    descendants.pop(descendants.index(descendant))  # get rid of the bad descendants
            bad_indicies = []  # find the indicies of the bad ones in the tax path
            for tax_id in tax_path:
                if tax_id in bad_keys:
                    index = tax_path.index(tax_id)
                    bad_indicies.append(index)  # store all the bad indicies
            bad_indicies.sort(reverse=True)  # in reverse order
            # branch_length += len(bad_indicies)  # increment the branch_lengths accordingly (might not work for good|bad|good|bad
            for index in bad_indicies:
                tax_path[index] = ''  # remove the bad tax_ids
            # fix the branch lengths and find the ancestors
            if len(tax_path) >= 2:
                ancestor = tax_path[-2]
                _data[key]["branch_length"] = 1
                i = -3
                while ancestor is "" or ancestor == key:  # if it's a blank or repeated, go up until finding ancestor
                    if i < -len(tax_path):  # Path is all the way full with bad tax_ids, connect to root
                        _data[key]["branch_length"] += 1
                        ancestor = "-1"
                        _data["-1"]["descendants"].append(key)  # note this is now a descendant of the root
                        break
                    else:
                        ancestor = tax_path[i]
                        _data[key]["branch_length"] += 1
                        i -= 1
                _data[key]["ancestor"] = ancestor
                if ancestor in _data:
                    if key not in _data[ancestor]["descendants"]:
                        _data[ancestor]["descendants"].append(key)  # Note this is now a descendant of it's ancestor
        return

    def _populate_missing_dont_use(self):
        # Unfortunately, some of the profile files may be missing intermediate ranks,
        # so we should manually populate them here
        # This will only really fix one missing intermediate rank....
        # INSTEAD!!! Let's just delete the missing intermediate ranks, if the tax_id isn't a key, delete it from the tax_id_path
        # This is really only useful for MetaPhlAn profiles, which due to the infrequently updated taxonomy, contain
        # many missing internal nodes
        _data = self._data
        for key in _data.keys():
            if "abundance" not in _data[key]:  # this is a missing intermediate rank
                # all the descendants *should* be in there, so leverage this info
                if "descendants" not in _data[key]:
                    print("You're screwed, malformed profile file with rank %s" % key)
                    raise Exception
                else:
                    descendant = _data[key]["descendants"][0]
                    to_populate_key = descendant  # just need the first one since the higher up path will be the same
                    to_populate = copy.deepcopy(_data[to_populate_key])
                    tax_path = to_populate["tax_path"]
                    tax_path_sn = to_populate["tax_path_sn"]
                    descendant_pos = tax_path.index(descendant)
                    for i in range(len(tax_path) - 1, descendant_pos - 1, -1):
                        tax_path.pop(i)
                        tax_path_sn.pop(i)
                    to_populate["branch_length"] = 1
                    if "rank" in to_populate:
                        rank = to_populate["rank"]
                        if rank == "strain":
                            rank = "species"
                        elif rank == "species":
                            rank = "genus"
                        elif rank == "genus":
                            rank = "family"
                        elif rank == "family":
                            rank = "order"
                        elif rank == "order":
                            rank = "class"
                        elif rank == "class":
                            rank = "phylum"
                        elif rank == "phylum":
                            rank = "superkingdom"
                        else:
                            print('Invalid rank')
                            raise Exception
                    else:
                        print('Missing rank')
                        raise Exception
                    to_populate["ancestor"] = tax_path[-2]
                    to_populate["rank"] = rank
                    # Now go through and sum up the abundance for which this guy is the ancestor
                    to_populate["abundance"] = 0
                    to_populate["descendants"] = []
                    for temp_key in _data.keys():
                        if "ancestor" in _data[temp_key]:
                            if _data[temp_key]["ancestor"] == key:
                                to_populate["abundance"] += _data[temp_key]["abundance"]
                                to_populate["descendants"].append(temp_key)
                    # Make sure this guy is listed as a descendant to his ancestor
                    if key not in _data[to_populate["ancestor"]]["descendants"]:
                        _data[to_populate["ancestor"]]["descendants"].append(key)
                    _data[key] = to_populate
        return

    def write_file(self, out_file_name=None):
        if out_file_name is None:
            raise Exception
        _data = self._data
        keys = _data.keys()
        # This will be annoying to keep things in order...
        # Let's iterate on the length of the tax_path since we know that will be in there
        tax_path_lengths = max([len(_data[key]["tax_path"]) for key in keys])
        fid = open(out_file_name, 'w')
        # Write the header
        for head in self._header:
            fid.write("%s\n" % head)

        # Loop over length of tax_path and write data
        # always make the output tax_id, rank, tax_path, tax_path_sn, abundance in that order
        for path_length in range(1, tax_path_lengths + 1):
            for key in keys:
                if len(_data[key]["tax_path"]) == path_length and _data[key]["abundance"] > self._eps:
                    line_data = _data[key]
                    fid.write("%s\t" % key)
                    if self._rank_pos is not None:
                        fid.write("%s\t" % line_data["rank"])
                    fid.write("%s\t" % "|".join(line_data["tax_path"]))
                    if self._tax_path_sn_pos is not None:
                        fid.write("%s\t" % "|".join(line_data["tax_path_sn"]))
                    fid.write("%f\n" % line_data["abundance"])
        fid.close()
        return

    def threshold(self, threshold=None):
        if threshold is None:
            raise Exception
        _data = self._data
        keys = _data.keys()
        for key in keys:
            if _data[key]["abundance"] < threshold:
                _data[key]["abundance"] = 0
        return

    def _subtract_down(self):
        # helper function to push all the weights up by subtracting
        # NOTE: when subtracting, need to start at root and go down
        # NOTE: when adding, need to start at leaves and go up
        _data = self._data
        keys = _data.keys()
        # This will be annoying to keep things in order...
        # Let's iterate on the length of the tax_path since we know that will be in there
        tax_path_lengths = max([len(_data[key]["tax_path"]) for key in keys])
        for path_length in range(1, tax_path_lengths):  # eg tax_path_lengths = 5, use 1,2,3,4 since we stop at leaves
            for key in keys:
                if len(_data[key]["tax_path"]) == path_length:
                    descendants = _data[key]["descendants"]  # get all descendants
                    for descendant in descendants:
                        _data[key]["abundance"] -= _data[descendant]["abundance"]  # subtract the descendants abundance

    def _add_up(self):
        # helper function to push all the weights up by subtracting
        # NOTE: when subtracting, need to start at root and go down
        # NOTE: when adding, need to start at leaves and go up
        _data = self._data
        keys = _data.keys()
        # This will be annoying to keep things in order...
        # Let's iterate on the length of the tax_path since we know that will be in there
        tax_path_lengths = max([len(_data[key]["tax_path"]) for key in keys])
        for path_length in range(tax_path_lengths, 1,
                                 -1):  # eg tax_path_lengths = 5, use 5,4,3,2, since we stop at roots
            for key in keys:
                if len(_data[key]["tax_path"]) == path_length:
                    ancestor = _data[key]["ancestor"]
                    if ancestor in _data:  # don't do anything if this is a/the root node
                        _data[ancestor]["abundance"] += _data[key]["abundance"]  # add the descendants abundance

    def normalize(self):
        # Need to really push it up while subtracting, then normalize, then push up wile adding
        # self._push_up(operation="subtract")
        self._subtract_down()
        _data = self._data
        keys = _data.keys()
        total_abundance = 0
        for key in keys:
            total_abundance += _data[key]["abundance"]
        # print(total_abundance)
        for key in keys:
            if total_abundance > 0:
                _data[key]["abundance"] /= total_abundance
                _data[key]["abundance"] *= 100  # make back into a percentage
        # self._push_up(operation="add")
        self._add_up()
        return

    def merge(self, other):
        # Warning: not checking for taxonomic consistency
        if not isinstance(other, Profile):
            print("Only works with other Profiles")
            raise Exception
        if self._merged_flag is False:
            self._header.insert(0, "# This is a merged file, ignore files in headers below")
            self._merged_flag = True
        _data = self._data
        _other_data = other._data
        other_keys = _other_data.keys()
        for key in other_keys:
            if key in _data:
                _data[key]["abundance"] += _other_data[key]["abundance"]  # if already in there, add abundances
            else:
                _data[key] = copy.copy(_other_data[key])  # otherwise use the whole thing

    def make_unifrac_input_and_normalize(self, other):
        if not isinstance(other, Profile):
            raise Exception
        _data = self._data
        _other_data = other._data

        _data_keys = _data.keys()
        tax_path_lengths1 = max([len(_data[key]["tax_path"]) for key in _data_keys])
        _other_data_keys = _other_data.keys()
        tax_path_lengths2 = max([len(_other_data[key]["tax_path"]) for key in _other_data_keys])
        tax_path_lengths = max(tax_path_lengths1, tax_path_lengths2)
        all_keys = set(_data_keys)
        all_keys.update(_other_data_keys)
        nodes_in_order = []
        for path_length in range(tax_path_lengths, 0, -1):
            for key in all_keys:
                if key in _data:
                    if len(_data[key]["tax_path"]) == path_length:
                        if key not in nodes_in_order:
                            nodes_in_order.append(key)
                elif key in _other_data:
                    if len(_other_data[key]["tax_path"]) == path_length:
                        if key not in nodes_in_order:
                            nodes_in_order.append(key)
        # Make the graph
        # Put the root at the very end
        if '-1' in nodes_in_order:
            nodes_in_order.pop(nodes_in_order.index('-1'))
            nodes_in_order.append('-1')
        else:
            nodes_in_order.append('-1')
        Tint = dict()
        lint = dict()
        for key in nodes_in_order:
            if key in _data:
                if "ancestor" in _data[key]:  # If ancestor is not in there, then it's an ancestor
                    ancestor = _data[key]["ancestor"]
                    Tint[key] = ancestor
                    lint[key, ancestor] = _data[key]["branch_length"]
            elif key in _other_data:
                if "ancestor" in _other_data[key]:
                    ancestor = _other_data[key]["ancestor"]
                    Tint[key] = ancestor
                    lint[key, ancestor] = _other_data[key]["branch_length"]
        nodes_to_index = dict(zip(nodes_in_order, range(len(nodes_in_order))))
        # return Tint, lint, nodes_in_order, nodes_to_index

        # Now need to change over to the integer-based indexing
        Tint2 = dict()
        lint2 = dict()
        nodes_in_order2 = []
        for key in nodes_in_order:
            if key in Tint:
                ancestor = Tint[key]
                Tint2[nodes_to_index[key]] = nodes_to_index[ancestor]
                if (key, ancestor) in lint:
                    lint2[nodes_to_index[key], nodes_to_index[ancestor]] = lint[key, ancestor]
            nodes_in_order2.append(nodes_to_index[key])

        # Next make the probability distributions
        # Would be nice if I could find a non-destructive way to subtract up and normalize

        # Do it for P
        self._subtract_down()
        keys = _data.keys()
        total_abundance = 0
        for key in keys:
            total_abundance += _data[key]["abundance"]
        # print(total_abundance)
        for key in keys:
            if total_abundance > 0:
                _data[key]["abundance"] /= total_abundance  # Should be a fraction, summing to 1
        P = np.zeros(len(nodes_in_order))
        for key_ind in range(len(nodes_in_order)):
            key = nodes_in_order[key_ind]
            if key in _data:
                P[key_ind] = _data[key]["abundance"]

        # Make back into percentages and add the mass back up (effectively normalizing the vector)
        for key in keys:
            if total_abundance > 0:
                _data[key]["abundance"] *= 100
        self._add_up()

        # Next do for Q
        other._subtract_down()
        keys = _other_data.keys()
        total_abundance = 0
        for key in keys:
            total_abundance += _other_data[key]["abundance"]
        # print(total_abundance)
        for key in keys:
            if total_abundance > 0:
                _other_data[key]["abundance"] /= total_abundance  # should be a fraction, summing to 1
        Q = np.zeros(len(nodes_in_order))
        for key_ind in range(len(nodes_in_order)):
            key = nodes_in_order[key_ind]
            if key in _other_data:
                Q[key_ind] = _other_data[key]["abundance"]

        # Make back into percentages and add the mass back up (effectively normalizing the vector)
        for key in keys:
            if total_abundance > 0:
                _other_data[key]["abundance"] *= 100
        other._add_up()

        return Tint2, lint2, nodes_in_order2, nodes_to_index, P, Q


def test_normalize():
    profile = Profile('/home/dkoslicki/Dropbox/Repositories/CAMIProfilingTools/src/test1.profile')
    profile.write_file('/home/dkoslicki/Dropbox/Repositories/CAMIProfilingTools/src/test1.profile.import')
    profile.normalize()
    profile.write_file('/home/dkoslicki/Dropbox/Repositories/CAMIProfilingTools/src/test1.profile.normalize')
    return profile


def test_unifrac():
    sys.path.append('/home/dkoslicki/Dropbox/Repositories/EMDUnifrac/src')
    import EMDUnifrac as EMDU
    profile1 = Profile('/home/dkoslicki/Dropbox/Repositories/EMDUnifrac/data/test1.profile')
    profile2 = Profile('/home/dkoslicki/Dropbox/Repositories/EMDUnifrac/data/test2.profile')
    t0 = timeit.default_timer()
    (Tint, lint, nodes_in_order, nodes_to_index, P, Q) = profile1.make_unifrac_input_and_normalize(profile2)
    t1 = timeit.default_timer()
    # print(t1-t0)
    (Z, diffab) = EMDU.EMDUnifrac_weighted(Tint, lint, nodes_in_order, P, Q)
    print(Z)
    (Tint, lint, nodes_in_order, nodes_to_index, P, Q) = profile1.make_unifrac_input_and_normalize(profile2)
    (Z2, diffab2) = EMDU.EMDUnifrac_weighted(Tint, lint, nodes_in_order, P, Q)
    print(Z2)
    profile1.normalize()
    profile2.normalize()
    (Tint, lint, nodes_in_order, nodes_to_index, P, Q) = profile1.make_unifrac_input_and_normalize(profile2)
    (Z3, diffab3) = EMDU.EMDUnifrac_weighted(Tint, lint, nodes_in_order, P, Q)
    print(Z3)

    print(diffab)
    print(diffab2)
    print(diffab3)


def test_real_data():
    profile1 = Profile(
        '/home/dkoslicki/Dropbox/Repositories/CAMIProfilingTools/src/lane4-s041-indexN722-S502-ATGCGCAG-ATAGAGAG-41_M5-2_S41_L004_R1_001.fa.gz.metaphlan.profile')
    profile2 = Profile(
        '/home/dkoslicki/Dropbox/Repositories/CAMIProfilingTools/src/lane8-s092-indexN729-S505-TCGACGTC-CTCCTTAC-91_Z0299_S92_L008_R1_001.fa.gz.metaphlan.profile')
    sys.path.append('/home/dkoslicki/Dropbox/Repositories/EMDUnifrac/src')
    import EMDUnifrac as EMDU
    (Tint, lint, nodes_in_order, nodes_to_index, P, Q) = profile1.make_unifrac_input_and_normalize(profile2)
    (Z, diffab) = EMDU.EMDUnifrac_weighted(Tint, lint, nodes_in_order, P, Q)
    print(Z)
    (Tint, lint, nodes_in_order, nodes_to_index, P, Q) = profile1.make_unifrac_input_and_normalize(profile2)
    (Z2, diffab2) = EMDU.EMDUnifrac_weighted(Tint, lint, nodes_in_order, P, Q)
    print(Z2)
    profile1.normalize()
    profile2.normalize()
    (Tint, lint, nodes_in_order, nodes_to_index, P, Q) = profile1.make_unifrac_input_and_normalize(profile2)
    (Z3, diffab3) = EMDU.EMDUnifrac_weighted(Tint, lint, nodes_in_order, P, Q)
    print(Z3)

    print(diffab)
    print(diffab2)
    print(diffab3)

# return profile1, profile2



#!/usr/bin/env python

# import biom
import os
import logging
from collections import defaultdict


class Prediction:
    def __init__(self):
        pass

    @property
    def rank(self):
        return self.__rank

    @property
    def taxid(self):
        return self.__taxid

    @property
    def percentage(self):
        return self.__percentage

    @property
    def taxpath(self):
        return self.__taxpath

    @property
    def taxpathsn(self):
        return self.__taxpathsn

    @rank.setter
    def rank(self, rank):
        self.__rank = rank

    @taxid.setter
    def taxid(self, taxid):
        self.__taxid = taxid

    @percentage.setter
    def percentage(self, percentage):
        self.__percentage = percentage

    @taxpath.setter
    def taxpath(self, taxpath):
        self.__taxpath = taxpath

    @taxpathsn.setter
    def taxpathsn(self, taxpathsn):
        self.__taxpathsn = taxpathsn

    def get_dict(self):
        return self.__dict__

    def get_pretty_dict(self):
        return {property.split("_")[3]: value for property, value in self.__dict__.items()}

    def get_metadata(self):
        return {'rank': self.__rank, 'taxpath': self.__taxpath, 'taxpathsn': self.__taxpathsn}


def get_column_indices(column_name_to_index):
    if "TAXID" not in column_name_to_index:
        logging.getLogger('opal').critical("Column not found: {}".format("TAXID"))
        raise RuntimeError
    if "RANK" not in column_name_to_index:
        logging.getLogger('opal').critical("Column not found: {}".format("RANK"))
        raise RuntimeError
    if "PERCENTAGE" not in column_name_to_index:
        logging.getLogger('opal').critical("Column not found: {}".format("PERCENTAGE"))
        raise RuntimeError
    if "TAXPATH" not in column_name_to_index:
        logging.getLogger('opal').critical("Column not found: {}".format("TAXPATH"))
        raise RuntimeError
    index_taxid = column_name_to_index["TAXID"]
    index_rank = column_name_to_index["RANK"]
    index_percentage = column_name_to_index["PERCENTAGE"]
    index_taxpath = column_name_to_index["TAXPATH"]
    if "TAXPATHSN" in column_name_to_index:
        index_taxpathsn = column_name_to_index["TAXPATHSN"]
    else:
        index_taxpathsn = None
    return index_rank, index_taxid, index_percentage, index_taxpath, index_taxpathsn


def normalize_samples(samples_list):
    for sample in samples_list:
        sample_id, sample_metadata, profile = sample
        sum_per_rank = defaultdict(float)
        for prediction in profile:
            sum_per_rank[prediction.rank] += prediction.percentage
        for prediction in profile:
            if prediction.percentage > 0:
                prediction.percentage = (prediction.percentage / sum_per_rank[prediction.rank]) * 100.0


def open_profile_from_tsv(file_path, normalize):
    header = {}
    column_name_to_index = {}
    profile = []
    samples_list = []
    predictions_dict = {}
    reading_data = False
    got_column_indices = False

    with open(file_path) as read_handler:
        for line in read_handler:
            if len(line.strip()) == 0 or line.startswith("#"):
                continue
            line = line.rstrip('\n')

            # parse header with column indices
            if line.startswith("@@"):
                for index, column_name in enumerate(line[2:].split('\t')):
                    column_name_to_index[column_name] = index
                index_rank, index_taxid, index_percentage, index_taxpath, index_taxpathsn = get_column_indices(column_name_to_index)
                got_column_indices = True
                reading_data = False
                continue

            # parse header with metadata
            if line.startswith("@"):
                # if last line contained sample data and new header starts, store profile for sample
                if reading_data:
                    if 'SAMPLEID' in header and 'VERSION' in header and 'RANKS' in header:
                        if len(profile) > 0:
                            samples_list.append((header['SAMPLEID'], header, profile))
                            profile = []
                            predictions_dict = {}
                    else:
                        logging.getLogger('opal').critical("Header in file {} is incomplete. Check if the header of each sample contains at least SAMPLEID, VERSION, and RANKS.\n".format(file_path))
                        raise RuntimeError
                    header = {}
                reading_data = False
                got_column_indices = False
                key, value = line[1:].split(':', 1)
                header[key.upper()] = value.strip()
                continue

            if not got_column_indices:
                logging.getLogger('opal').critical("Header line starting with @@ in file {} is missing or at wrong position.\n".format(file_path))
                raise RuntimeError

            reading_data = True
            row_data = line.split('\t')

            taxid = row_data[index_taxid]
            # if there is already a prediction for taxon, only sum abundance
            if taxid in predictions_dict:
                prediction = predictions_dict[taxid]
                prediction.percentage += float(row_data[index_percentage])
            else:
                prediction = Prediction()
                predictions_dict[taxid] = prediction
                prediction.taxid = row_data[index_taxid]
                prediction.rank = row_data[index_rank]
                prediction.percentage = float(row_data[index_percentage])
                prediction.taxpath = row_data[index_taxpath]
                if isinstance(index_taxpathsn, int):
                    prediction.taxpathsn = row_data[index_taxpathsn]
                else:
                    prediction.taxpathsn = None
                profile.append(prediction)

    # store profile for last sample
    if 'SAMPLEID' in header and 'VERSION' in header and 'RANKS' in header:
        if reading_data and len(profile) > 0:
            samples_list.append((header['SAMPLEID'], header, profile))
    else:
        logging.getLogger('opal').critical("Header in file {} is incomplete. Check if the header of each sample contains at least SAMPLEID, VERSION, and RANKS.\n".format(file_path))
        raise RuntimeError

    if normalize:
        normalize_samples(samples_list)

    return samples_list


def open_profile(file_path, normalize):
    if not os.path.exists(file_path):
        logging.getLogger('opal').critical("Input file {} does not exist.".format(file_path))
        exit(1)
    try:
        return open_profile_from_tsv(file_path, normalize)
    except:
        logging.getLogger('opal').critical("Input file could not be read.")
        exit(1)

    # try:
    #     table = biom.load_table(file_path)
    # except:
    #     try:
    #         return open_profile_from_tsv(file_path, normalize)
    #     except:
    #         logging.getLogger('opal').critical("Input file could not be read.")
    #         exit(1)
    #
    # samples_list = []
    # samples = table.ids(axis='sample')
    # ids = table.ids(axis='observation')
    #
    # for sample_id in samples:
    #     sample_metadata = table.metadata(id=sample_id, axis='sample')
    #     profile = []
    #     for id in ids:
    #         prediction = Prediction()
    #         metadata = table.metadata(id=id, axis='observation')
    #         prediction.taxid = id
    #         prediction.rank = metadata['rank']
    #         prediction.percentage = float(table.get_value_by_ids(obs_id=id, samp_id=sample_id))
    #         prediction.taxpath = metadata['taxpath']
    #         prediction.taxpathsn = metadata['taxpathsn']
    #         profile.append(prediction)
    #     samples_list.append((sample_id, sample_metadata, profile))
    #
    # if normalize:
    #     normalize_samples(samples_list)
    #
    # return samples_list


def get_taxa_names(profile):
    tax_id_to_name = {}
    for prediction in profile:
        tax_id_to_name[prediction.taxid] = prediction.taxpathsn.split("|")[-1]
    return tax_id_to_name


def get_rank_to_taxid_to_percentage(profile, rank=None):
    rank_to_taxid_to_percentage = {}
    for prediction in profile:
        if rank and prediction.rank != rank:
            continue
        if prediction.rank not in rank_to_taxid_to_percentage:
            rank_to_taxid_to_percentage[prediction.rank] = {}
        rank_to_taxid_to_percentage[prediction.rank][prediction.taxid] = prediction.percentage
    return rank_to_taxid_to_percentage



#!/usr/bin/env python

import math
from collections import OrderedDict
from src.utils import constants as c


class Shannon:
    def __init__(self, rank):
        self.__rank = rank

    @property
    def rank(self):
        return self.__rank

    @property
    def diversity(self):
        return self.__diversity

    @property
    def equitability(self):
        return self.__equitability

    @rank.setter
    def rank(self, rank):
        self.__rank = rank

    @diversity.setter
    def diversity(self, diversity):
        self.__diversity = diversity

    @equitability.setter
    def equitability(self, equitability):
        self.__equitability = equitability

    def get_dict(self):
        return self.__dict__

    def get_pretty_dict(self):
        return {metric.split("_")[3]: value for metric, value in self.__dict__.items()}


# Compute the Shannon index (diversity and equitability) with log base e
# The higher the index, the more equally distributed the taxa are
def compute_shannon_index(rank_to_taxid_to_percentage):
    rank_to_shannon = OrderedDict()
    for rank in c.ALL_RANKS:
        if rank not in rank_to_taxid_to_percentage:
            continue
        rank_to_shannon[rank] = Shannon(rank)
        rank_to_shannon[rank].diversity = .0
        rank_to_shannon[rank].equitability = .0
        for taxid in rank_to_taxid_to_percentage[rank]:
            percentage = rank_to_taxid_to_percentage[rank][taxid] / 100.0
            if percentage > .0:
                rank_to_shannon[rank].diversity -= percentage * math.log(percentage)
        if len(rank_to_taxid_to_percentage[rank]) > 1:
            rank_to_shannon[rank].equitability = rank_to_shannon[rank].diversity / math.log(len(rank_to_taxid_to_percentage[rank]))
    return rank_to_shannon



#!/usr/bin/env python3

import docker
import time
import argparse
import os
import sys
from opal import make_sure_path_exists
from version import __version__


def get_image_dir_name(image):
    return image.replace('/', '-').replace(':', '-')


def run_docker(image, volumes, yaml):
    client = docker.from_env()
    container = client.containers.run(image=image, command=None, volumes=volumes, detach=True, environment=yaml)
    stats = container.stats(stream=True, decode=True)
    max_total_rss = .0

    while not container.status.startswith("e"):
        container.reload()
        next_stats = next(stats)
        time.sleep(1)
        try:
            total_rss = next_stats["memory_stats"]["stats"]["total_rss"]
            if total_rss > max_total_rss:
                max_total_rss = total_rss
        except KeyError:
            pass
    return max_total_rss


def main():
    parser = argparse.ArgumentParser(description='Run biobox of profiler and track runtime and main memory usage', add_help=False)
    group1 = parser.add_argument_group('required arguments')
    group1.add_argument('image', nargs=1, help='Docker image (biobox) of profiler')
    group1.add_argument('--input_dir', help='Input directory containing gzipped FASTQ files', required=True)
    group1.add_argument('--output_dir', help='Output directory', required=True)
    group2 = parser.add_argument_group('optional arguments')
    group2.add_argument('--yaml', help='Bioboxes YAML file (default: INPUT_DIR/biobox.yaml)', required=False)
    group2.add_argument("--volume", help='Docker volume', type=lambda x: x.split(':'), action='append')
    group2.add_argument('-v', '--version', action='version', version='%(prog)s ' + __version__)
    group2.add_argument('-h', '--help', action='help', help='Show this help message and exit')
    args = parser.parse_args()
    input_dir = args.input_dir
    output_dir = args.output_dir
    image_dir_name = get_image_dir_name(args.image[0])

    volumes = dict()
    yaml = None
    if args.yaml:
        if os.path.dirname(args.yaml) == input_dir:
            yaml = {'YAML': '/bbx/mnt/input/' + os.path.basename(args.yaml)}
        else:
            yaml = {'YAML': '/bbx/mnt/yaml/' + os.path.basename(args.yaml)}
            volumes[os.path.dirname(args.yaml)] = {'bind': '/bbx/mnt/yaml', 'mode': 'ro'}

    volumes[input_dir] = {'bind': '/bbx/mnt/input', 'mode': 'ro'}
    volumes[os.path.join(output_dir, image_dir_name)] = {'bind': '/bbx/mnt/output', 'mode': 'rw'}
    volumes[os.path.join(output_dir, image_dir_name, 'metadata')] = {'bind': '/bbx/metadata', 'mode': 'rw'}
    volumes[os.path.join(output_dir, image_dir_name, 'cache')] = {'bind': '/cache', 'mode': 'rw'}
    if args.volume:
        for volume in args.volume:
            if len(volume) < 3:
                sys.exit("Invalid syntax for --volume")
            volumes[volume[0]] = {'bind': volume[1], 'mode': volume[2]}

    make_sure_path_exists(output_dir)
    make_sure_path_exists(os.path.join(output_dir, image_dir_name))
    make_sure_path_exists(os.path.join(output_dir, image_dir_name, 'metadata'))
    make_sure_path_exists(os.path.join(output_dir, image_dir_name, 'cache'))

    start_time = time.time()
    max_total_rss = run_docker(args.image[0], volumes, yaml)
    elapsed_time = time.time() - start_time
    memory = max_total_rss / 1048576.0

    print('{:.2f} seconds ({:.2f} hours)\n{} MB ({} GB)'.format(elapsed_time, elapsed_time/3600, memory, memory/1024))

    with open(os.path.join(output_dir, image_dir_name, 'runtime_maxmemory.txt'), 'w') as f:
        f.write('{:.2f} seconds ({:.2f} hours)\n{} MB ({} GB)\n'.format(elapsed_time, elapsed_time/3600, memory, memory/1024))


if __name__ == "__main__":
    main()



#!/usr/bin/env python
import docker
import time
import argparse


def run_docker(image, volumes, command, environment):
    client = docker.from_env()
    container = client.containers.run(image, command, volumes=volumes, detach=True, environment=environment)
    stats = container.stats(stream=True, decode=True)
    max_usage = .0
    max_usage_test = .0
    max_total_rss = .0

    while not container.status.startswith("e"):
        container.reload()
        next_stats = next(stats)
        # print json.dumps(next_stats, sort_keys=True, indent=4, separators=(',', ': '))
        time.sleep(1)
        try:
            max_usage = next_stats["memory_stats"]["max_usage"]
            usage = next_stats["memory_stats"]["usage"]
            if usage > max_usage_test:
                max_usage_test = usage
            total_rss = next_stats["memory_stats"]["stats"]["total_rss"]
            if total_rss > max_total_rss:
                max_total_rss = total_rss
        except KeyError:
            pass
    print "%f MB\t%f MB\t%f MB" % (max_usage / 1048576.0, max_usage_test / 1048576.0, max_total_rss / 1048576.0)

parser = argparse.ArgumentParser()
parser.add_argument("-i", "--image")
parser.add_argument("-v", "--volume", type=lambda x: x.split(":"), action="append")
parser.add_argument("-c", "--command")
parser.add_argument("-e", "--environment")
args = parser.parse_args()
if not args.image or not args.volume:
    parser.print_help()
    parser.exit(1)
if not args.command:
    run_command = None
else:
    run_command = args.command
volumes = dict()
for i in range(0, len(args.volume)):
    # print "%s %s %s" % (args.volume[i][0], args.volume[i][1], args.volume[i][2])
    if len(args.volume[0]) < 3:
        parser.print_help()
        exit(1)
    volumes[args.volume[i][0]] = {'bind': args.volume[i][1], 'mode': args.volume[i][2]}

#print volumes
#exit()
environment = None
if args.environment:
    environment = {'YAML': args.environment}

start_time = time.time()
run_docker(args.image, volumes, args.command, environment)
elapsed_time = time.time() - start_time
print "%.2fs or %.2fm or %.2fh" % (elapsed_time, elapsed_time/60, elapsed_time/3600)

